﻿using UnityEngine;
using System;
using System.Collections;
using System.Collections.Generic;

[Serializable]
public class LevelParams
{
	public int length;
	public int lowcount;
	public int highcount;
	public float wavebufferlow;
	public float wavebufferhigh;
	public float spread;
	public float variance;
	public int lowalien;
	public int highalien;
}

public class LevelGenerator {

	private GameController gc;

	public LevelGenerator(GameController g)
	{
		gc = g;
	}

	public LevelData GenerateLevel(int lev)
	{
		Debug.Log ("LevelGenerator Generating Level...");

		LevelParams lparams;
		if(lev < gc.LevelParamsList.Count)
			lparams = gc.LevelParamsList[lev];
		else 
			lparams = gc.LevelParamsList[gc.LevelParamsList.Count-1];

		LevelData level = new LevelData(lev);
		level.Length = lparams.length;
		level.StartPosition = new Vector2(0f,2.5f);

		level.WaterLinePrefabs.AddRange (gc.WaterLinePrefabs);
		level.FloorPrefab = gc.FloorPrefab;
		level.UnderwaterBasePrefab = gc.UnderwaterBasePrefab;
		level.UnderwaterOverlayPrefab = gc.UnderwaterOverlayPrefab;

		float x;
		if(lev==1)
			x = 45f;
		else if(lev<5)
			x = 35;
		else if(lev<10)
			x = 30;
		else if(lev<15)
			x = 25;
		else 
			x = 20;

		while(x < (level.Length-25))
		{
			int count = gc.rand.Next(lparams.lowcount,lparams.highcount+1);
			AddWave(level, x, count, lparams);
			//x += gc.rand.Next(lparams.wavebufferlow,lparams.wavebufferhigh+1) + count * (lparams.spread + 0.6f*lparams.variance);
			float buffer = lparams.wavebufferlow + (float)gc.rand.NextDouble() * (lparams.wavebufferhigh - lparams.wavebufferlow);
			x += buffer + count * (lparams.spread + 0.6f*lparams.variance);
		}

		return level;
	}

	public void AddWave(LevelData level, float xloc, int count, LevelParams lparams)
	{
		Debug.Log ("LevelGenerator       Adding Wave of " + count + " at " + xloc);

		WaveData wave = new WaveData();
		wave.XLocation = xloc;
		wave.Spread = lparams.spread;
		wave.Variance = lparams.variance;

		for(int i = 0; i<count; i++)
		{
			int berg = gc.rand.Next(0, gc.IcebergPrefabs.Count);  // TODO this was hi-# exclusive, then started being inclusive?!??

			Debug.Log ("LevelGenerator                Adding Iceberg " + (i+1) + " with prefab index " + berg + " to wave");

			wave.ObstaclePrefabs.Add(gc.IcebergPrefabs[berg]);
		}

		if(lparams.highalien > -1)
		{
			int alien = gc.rand.Next(lparams.lowalien, (int)Mathf.Min(lparams.highalien+1, gc.AlienPrefabs.Count+1));
			wave.AlienPrefabs.Add(gc.AlienPrefabs[alien]);
			level.AlienCount++;
		}

		level.Waves.Add(wave);

	}
	

}
