﻿using UnityEngine;
using System.Collections;

public class PlayerMovementController : MonoBehaviour {

	public float ForwardAirSpeed = 16;
	public float ForwardAirAccel = 8;
	public float ForwardAirSlow = 8;

	public float ForwardWaterSpeed = 16;
	public float ForwardWaterAccel = 8;
	public float ForwardWaterSlow = 8;

	public float ThrustForce = 1000f;
	public float ThrustSoundReplayInterval = 0.25f;

	public float Gravity = -9.8f;
	public float Bouyancy = 6.2f;

	public Vector2 Moving { get; set; }
	public bool ThrustingUp { get; set; }
	public bool ThrustingDown { get; set; }

	private Transform upThrust;
	private Transform downThrust;

	private float stunnedUntil;
	private bool stunned;
	private float lastThrustSound;

	private Animator animator;
	private GameController gc;

	public void Stun(float seconds)
	{
		Debug.Log ("Player Stunned for " + seconds);
		stunnedUntil = Time.time + seconds;
	}

	void Start () {
		animator = GetComponent<Animator>();

		GameObject go = GameObject.Find ("GameController");
		gc = go.GetComponent<GameController>();

		upThrust = transform.Find ("UpThrust");
		downThrust = transform.Find ("DownThrust");

		upThrust.renderer.enabled = false;
		downThrust.renderer.enabled = false;
	}
	
	void FixedUpdate () {
		stunned = !(Time.time > stunnedUntil);
		animator.SetBool ("Stunned", stunned);

		if(transform.position.y > 0)
			AirMove ();
		else
			WaterMove ();

		if(stunned)
		{
			upThrust.renderer.enabled = false;
			downThrust.renderer.enabled = false;
		}
		else
		{
			if(ThrustingUp)
			{
				upThrust.renderer.enabled = true;
				rigidbody2D.AddForce(new Vector2(0, ThrustForce));
				ThrustSound();
			}
			else 
			{
				upThrust.renderer.enabled = false;
			}

			if(ThrustingDown)
			{
				downThrust.renderer.enabled = true;
				ThrustSound();

				// problem with player staying underwater, didn't want to totally gimp jets, so here's a hack?
				if(transform.position.y < -0.2f)
				{
					float buoyantforce = rigidbody2D.mass * Bouyancy;
					rigidbody2D.AddForce(new Vector2(0, -buoyantforce*0.8f));
				}
				else
				{
					rigidbody2D.AddForce(new Vector2(0, -ThrustForce));
				}
			}
			else 
			{
				downThrust.renderer.enabled = false;
			}
		}
	}

	private void ThrustSound()
	{
		if(Time.time - lastThrustSound > ThrustSoundReplayInterval)
		{
			lastThrustSound = Time.time;
			if(transform.position.y > 0)
				gc.PlaySound(gc.ThrustSoundAir, 0.5f);
			else
				gc.PlaySound(gc.ThrustSoundUnderwater, 0.75f);
		}
	}

	private void AirMove()
	{
		ApplyForwardAirThrust();
		ApplyGravity();
	}

	private void WaterMove()
	{
		ApplyForwardWaterThrust();
		ApplyBouyancy();
	}

	private void ApplyGravity()
	{
		rigidbody2D.AddForce(new Vector2(0, rigidbody2D.mass * Gravity));
	}

	private void ApplyBouyancy()
	{
		rigidbody2D.AddForce(new Vector2(0, rigidbody2D.mass * Bouyancy));
	}

	private void ApplyForwardAirThrust()
	{
		float deltaX = ForwardAirSpeed - rigidbody2D.velocity.x;
		float accel = 0;

		if(deltaX < 0) // somehow got above max speed, slow down
		{
			accel = Mathf.Max(deltaX, -ForwardAirSlow);
		}

		if(deltaX > 0) // thrust toward max speed
		{
			accel = Mathf.Min(deltaX, ForwardAirAccel);
		}

		float xforce = rigidbody2D.mass * accel / Time.deltaTime;
		rigidbody2D.AddForce(new Vector2(xforce, 0));
	}

	private void ApplyForwardWaterThrust()
	{
		float deltaX = ForwardWaterSpeed - rigidbody2D.velocity.x;
		float accel = 0;
		
		if(deltaX < 0) // somehow got above max speed, slow down
		{
			accel = Mathf.Max(deltaX, -ForwardWaterSlow);
		}
		
		if(deltaX > 0) // thrust toward max speed
		{
			accel = Mathf.Min(deltaX, ForwardWaterAccel);
		}
		
		float xforce = rigidbody2D.mass * accel / Time.deltaTime;
		rigidbody2D.AddForce(new Vector2(xforce, 0));
	}

}
